-- ///////////////////////////////////////////////////////////////////////////
-- Modify this line to correct DSM database name.
use [dsm]
go
-- ///////////////////////////////////////////////////////////////////////////
set nocount on
go
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Recovery Commands Preparing'
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @altercolumnspec nvarchar(3192) = 'systemevents.TargetID:BIGINT:null|antimalwareevents.AntiMalwareEventID:BIGINT|antimalwarequarantinedfile.AntiMalwareEventID:BIGINT|antimalwarespywareitems.AntiMalwareEventID:BIGINT:null|webreputationevents.WebReputationEventID:BIGINT|antimalwareeventhistory.AntiMalwareEventHistoryID:BIGINT|antimalwareeventhistory.AntiMalwareEventID:BIGINT:null|webreputationeventhistory.WebReputationEventHistoryID:BIGINT|webreputationeventhistory.WebReputationEventID:BIGINT:null|integrityevents.IntegrityEventID:BIGINT|integrityeventhistory.IntegrityEventHistoryID:BIGINT|integrityeventhistory.IntegrityEventID:BIGINT:null|loginspectionevents.LogInspectionEventID:BIGINT|loginspectioneventhistory.LogInspectionEventHistoryID:BIGINT|loginspectioneventhistory.LogInspectionEventID:BIGINT:null|packetlogs.PacketLogID:BIGINT|packetlogdatas.PacketLogDataID:BIGINT|payloadlogs.PayloadLogID:BIGINT|payloadlogdatas.PayloadLogDataID:BIGINT|packetloghistory.PacketLogHistoryID:BIGINT|packetloghistory.PacketLogID:BIGINT:null|payloadloghistory.PayloadLogHistoryID:BIGINT|payloadloghistory.PayloadLogID:BIGINT:null';
-- ///////////////////////////////////////////////////////////////////////////
declare @upgrade_spec table (pk_table_name nvarchar(255)
                            ,pk_column_name nvarchar(255)
                            ,pk_datatype nvarchar(255)
                            ,nullable nvarchar(3));
declare @altercolumnentry nvarchar(512);
declare @altercolumnspecidx int = len(@altercolumnspec);
while (@altercolumnspecidx > 0)
begin
    set @altercolumnspecidx = charindex('|', @altercolumnspec);
    if (@altercolumnspecidx = 0)
    begin
        set @altercolumnentry = @altercolumnspec;
        set @altercolumnspec = '';
    end
    else
    begin
        set @altercolumnentry = substring(@altercolumnspec,1,@altercolumnspecidx-1);
        set @altercolumnspec = substring(@altercolumnspec,@altercolumnspecidx+1,len(@altercolumnspec)-@altercolumnspecidx);
    end
    declare @tableEndPos int = charindex('.', @altercolumnentry);
    declare @columnEndPos int = charindex(':', @altercolumnentry);
    declare @altercolumnentry_tableName nvarchar(127) = substring(@altercolumnentry,1,@tableEndPos-1);
    declare @altercolumnentry_columnName nvarchar(127) = substring(@altercolumnentry,@tableEndPos+1,@columnEndPos-@tableEndPos-1);
    declare @altercolumnentry_datatype nvarchar(127) = substring(@altercolumnentry,@columnEndPos+1,len(@altercolumnentry)-@columnEndPos);
    declare @altercolumnentry_nullable nvarchar(3) = 'NO';
    if (substring(@altercolumnentry_datatype,len(@altercolumnentry_datatype)-4, 5) = ':null')
    begin
        set @altercolumnentry_nullable = 'YES';
        set @altercolumnentry_datatype = substring(@altercolumnentry_datatype,1, len(@altercolumnentry_datatype)-5);
    end
    insert into @upgrade_spec(pk_table_name, pk_column_name, pk_datatype, nullable)
    values(@altercolumnentry_tableName,@altercolumnentry_columnName,@altercolumnentry_datatype, @altercolumnentry_nullable);
end
-- ///////////////////////////////////////////////////////////////////////////
declare @dbname nvarchar(255);
declare @dbid nvarchar(255);
declare @schemaname nvarchar(255);
declare @schemaid nvarchar(255);
declare @editionID nvarchar(255);
declare @recovery_model nvarchar(255);
declare cur_dbinfo cursor for select db_name(), db_id(), schema_name(), schema_id(), cast(serverproperty('EditionID') as nvarchar(255));
open cur_dbinfo;
fetch next from cur_dbinfo into @dbname, @dbid, @schemaname, @schemaid, @editionID;
close cur_dbinfo;
deallocate cur_dbinfo;
declare cur_recovery_model cursor for select recovery_model_desc from sys.databases where name = @dbname;
open cur_recovery_model;
fetch next from cur_recovery_model into @recovery_model;
close cur_recovery_model;
deallocate cur_recovery_model;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking primary keys'
-- ///////////////////////////////////////////////////////////////////////////
declare @pk_info table (pk_name nvarchar(255)
                       ,table_name nvarchar(255)
                       ,column_name nvarchar(255)
                       ,datatype nvarchar(255));
insert into @pk_info (pk_name, table_name, column_name, datatype)
select distinct a.constraint_name, a.table_name, b.column_name, c.pk_datatype
from information_schema.table_constraints a, information_schema.constraint_column_usage b, @upgrade_spec c
where a.table_catalog = b.table_catalog and a.table_schema = b.table_schema
and a.table_name = b.table_name
and b.table_name = c.pk_table_name
and a.constraint_name = b.constraint_name
and a.constraint_type = 'PRIMARY KEY';
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking foreign keys and indexes'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_pk cursor for select pk_name, table_name, column_name, datatype from @pk_info;
declare @temp_pk_pk_name nvarchar(255);
declare @temp_pk_table_name nvarchar(255);
declare @temp_pk_column_name nvarchar(255);
declare @temp_pk_datatype nvarchar(255);
declare @fk_info table (fk_name nvarchar(255)
                       ,fk_table_name nvarchar(255)
                       ,fk_column_name nvarchar(255)
                       ,pk_name nvarchar(255)
                       ,pk_table_name nvarchar(255)
                       ,pk_column_name nvarchar(255)
                       ,fk_delete_rule nvarchar(20)
                       ,datatype nvarchar(255));
-- Note that this presumes that there is only one column per primary key (and thus foreign keys as well)
insert into @fk_info (fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype)
select distinct rc.constraint_name, lower(fk_tc.table_name), lower(fk_ccu.column_name), lower(rc.unique_constraint_name), lower(pk.table_name), lower(pk.column_name), rc.delete_rule, pk.datatype
from information_schema.referential_constraints rc
    ,@pk_info pk
    ,information_schema.table_constraints fk_tc
    ,information_schema.constraint_column_usage fk_ccu
where rc.unique_constraint_name = pk.pk_name
and rc.constraint_catalog = @dbname and rc.constraint_schema = @schemaname
and rc.constraint_catalog = fk_tc.constraint_catalog and rc.constraint_schema = fk_tc.constraint_schema and rc.constraint_name = fk_tc.constraint_name
and rc.constraint_catalog = fk_ccu.constraint_catalog and rc.constraint_schema = fk_ccu.constraint_schema and rc.constraint_name = fk_ccu.constraint_name
and fk_tc.constraint_catalog = fk_ccu.constraint_catalog and fk_tc.constraint_schema = fk_ccu.constraint_schema and fk_tc.constraint_name = fk_ccu.constraint_name;
declare @nonkey_indexes table (index_name nvarchar(255)
                              ,index_table nvarchar(255)
                              ,index_column nvarchar(255)
                              ,is_unique int
                              ,is_unique_constraint int
                              ,is_descending nvarchar(10)
                              ,type_desc nvarchar(255));
insert into @nonkey_indexes
select ind.name, t.name, col.name, ind.is_unique, ind.is_unique_constraint, case when ic.is_descending_key = 0 then 'ASC' else 'DESC' end, ind.type_desc
from   sys.indexes ind, sys.index_columns ic, sys.columns col, sys.tables t
where ind.is_primary_key = 0 and ind.object_id = ic.object_id and ind.index_id = ic.index_id
and ic.object_id = col.object_id and ic.column_id = col.column_id and ind.object_id = t.object_id
and t.is_ms_shipped = 0
and t.name + '|' + col.name in (select pku.pk_table_name + '|' + pku.pk_column_name from @upgrade_spec pku)
order by t.name, ind.name, ind.index_id, ic.index_column_id;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking all columns to be altered'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_nonkey_indexes cursor for select index_name, index_table, index_column, is_unique, is_unique_constraint, is_descending, type_desc from @nonkey_indexes;
declare @temp_nonkeyindex_name nvarchar(255);
declare @temp_nonkeyindex_table nvarchar(255);
declare @temp_nonkeyindex_column nvarchar(255);
declare @temp_nonkeyindex_is_unique int;
declare @temp_nonkeyindex_is_unique_constraint int;
declare @temp_nonkeyindex_is_descending nvarchar(10);
declare @temp_nonkey_type_desc nvarchar(255);
declare @alter_column_table table (table_name nvarchar(255)
                                  ,column_name nvarchar(255)
                                  ,datatype nvarchar(255)
                                  ,nullable nvarchar(3));
insert into @alter_column_table (table_name, column_name, datatype, nullable)
select pk_table_name, pk_column_name, pk_datatype, nullable from @upgrade_spec
union
select fk_tc.table_name, fk_ccu.column_name, upg.pk_datatype, upg.nullable
from information_schema.referential_constraints rc,
     information_schema.table_constraints fk_tc,
     information_schema.constraint_column_usage fk_ccu,
     information_schema.constraint_column_usage rcu,
     @upgrade_spec upg
where rc.constraint_catalog = @dbname and rc.constraint_schema = @schemaname
and rc.constraint_catalog = fk_tc.constraint_catalog and rc.constraint_schema = fk_tc.constraint_schema and rc.constraint_name = fk_tc.constraint_name
and rc.constraint_catalog = fk_ccu.constraint_catalog and rc.constraint_schema = fk_ccu.constraint_schema and rc.constraint_name = fk_ccu.constraint_name
and fk_tc.constraint_catalog = fk_ccu.constraint_catalog and fk_tc.constraint_schema = fk_ccu.constraint_schema and fk_tc.constraint_name = fk_ccu.constraint_name
and rc.unique_constraint_catalog = rcu.constraint_catalog and rc.unique_constraint_schema = rcu.constraint_schema and rc.unique_constraint_name = rcu.constraint_name
and lower(rcu.table_name + '|' + rcu.column_name) = lower(upg.pk_table_name + '|' + upg.pk_column_name);
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop foreign keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_alter_column_distinct_tables cursor for select distinct table_name from @alter_column_table;
declare @temp_alter_column_distinct_table_name nvarchar(255);
declare @temp_fk_name nvarchar(255);
declare @temp_fk_table nvarchar(255);
declare @temp_fk_column nvarchar(255);
declare @temp_fk_pk_table nvarchar(255);
declare @temp_fk_pk_column nvarchar(255);
declare @temp_fk_pk_name nvarchar(255);
declare @temp_fk_delete_rule nvarchar(255);
declare @temp_fk_datatype nvarchar(255);
declare cur_fk_tableInfo cursor for select distinct fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype from @fk_info;
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @drop_constraint_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' drop constraint ' + @temp_fk_name + ';';
    print '    ' + @drop_constraint_cmd;
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop other indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare @dropped_uniqueKeyConstraints table (constraint_name nvarchar(2000)
                                            ,table_name nvarchar(2000)
                                            ,column_name nvarchar(2000));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 1)
    begin
        declare @drop_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' drop constraint ' + @temp_nonkeyindex_name;
        print '    ' + @drop_unique_constraint_cmd;
        insert into @dropped_uniqueKeyConstraints(constraint_name, table_name, column_name)
        values( @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column);
    end;
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end;
close cur_nonkey_indexes;
declare @dropped_nonkeyIndexes table (index_name nvarchar(2000)
                                     ,type_desc nvarchar(2000)
                                     ,table_name nvarchar(2000)
                                     ,column_name nvarchar(2000)
                                     ,is_descending nvarchar(10));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 0)
    begin
        declare @drop_nonkey_index_cmd nvarchar(255) = 'drop index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + ';';
        print '    ' + @drop_nonkey_index_cmd;
        insert into @dropped_nonkeyIndexes(index_name, type_desc, table_name, column_name, is_descending)
        values( @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending);
    end
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end
close cur_nonkey_indexes;
deallocate cur_nonkey_indexes;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop primary keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @drop_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' drop constraint ' + @temp_pk_pk_name + ';';
    print '    ' + @drop_pk_cmd;
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate primary keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @create_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' add constraint ' + @temp_pk_pk_name + ' primary key(' + @temp_pk_column_name + ');';
    print '    ' + @create_pk_cmd;
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
deallocate cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate foreign keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @create_fk_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' with check add constraint ' + @temp_fk_name + ' foreign key(' + @temp_fk_column + ') references ' + @temp_fk_pk_table + '(' + @temp_fk_pk_column + ')';
    if (@temp_fk_delete_rule='CASCADE')
    begin
        set @create_fk_cmd = @create_fk_cmd + ' on delete cascade';
    end;
    set @create_fk_cmd = @create_fk_cmd + ';';
    print '    ' + @create_fk_cmd;
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
deallocate cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate uniquekeys for other non-primary key indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_uniqueKeyConstraints cursor for
select constraint_name, table_name, column_name from @dropped_uniqueKeyConstraints;
open cur_dropped_uniqueKeyConstraints;
fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
while @@FETCH_STATUS = 0
begin
    declare @create_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' add constraint ' + @temp_nonkeyindex_name + ' unique(' + @temp_nonkeyindex_column + ');';
    print '    ' + @create_unique_constraint_cmd;
    fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
end
close cur_dropped_uniqueKeyConstraints;
deallocate cur_dropped_uniqueKeyConstraints;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate any leftover nonkey indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_nonkeyIndexes cursor for
select index_name, type_desc, table_name, column_name, is_descending from @dropped_nonKeyIndexes;
open cur_dropped_nonkeyIndexes;
fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
while @@FETCH_STATUS = 0
begin
    declare @create_nonkeyindex_cmd nvarchar(255) = 'create ' + @temp_nonkey_type_desc + ' index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + '(' + @temp_nonkeyindex_column + + ' ' + @temp_nonkeyindex_is_descending + ');';
    print '    ' + @create_nonkeyindex_cmd;
    fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
end
close cur_dropped_nonkeyIndexes;
deallocate cur_dropped_nonkeyIndexes;
go
-- ///////////////////////////////////////////////////////////////////////////
set nocount off
go
